
def _maketranslator():
    '''
    Make a translator to sanitise names for python variable
    '''
    from string import ascii_letters, digits, printable, maketrans
    safechars = '_' + ascii_letters + digits
    trans = list(printable)
    for i,t in enumerate(trans):
        if not t in safechars:
            trans[i] = '_'
    tostrs = ''.join(trans)
    return maketrans(printable, tostrs)

_translator = _maketranslator()

def sanitise_name(text):
    '''
    Sanitise name
    '''
    if text == 'metadata':
        text += '_'
    from string import digits
    sane = text.translate(_translator)
    if sane[0] in digits:
        sane = '_' + sane
    return sane

def make_sane(items):
    '''
    '''
    if items is None or len(items) == 0:
        return []

    items = zip(*items)
    keys = [sanitise_name(str(k)) for k in items[0]]
    vals = items[1]

    lk = len(keys)
    sk = set(keys)
    ls = len(sk)
    if ls < lk:
        # some headers are not unique so append a number
        uk = dict(zip(sk, [0]*ls))
        for i,k in enumerate(keys):
            n = uk[k]
            if n > 0:
                newkey = keys[i] + str(n)
                while newkey in uk:
                    n += 1
                    newkey = keys[i] + str(n)
                keys[i] = newkey
            uk[k] = n+1
    
    return zip(keys, vals)

from _external.ordereddict import OrderedDict as _odict

class ListDict(_odict):
    '''
    Combined list/ordered dictionary class
    '''
    def __init__(self, data):
        '''
        List of tuples of key/value pairs
        '''
        data = make_sane(data)
        _odict.__init__(self, data)
        self.__dict__.update(data)

    def __getitem__(self, key):
        from types import StringType, IntType
        if type(key) is IntType:
            key = self.keys()[key]

        if type(key) is StringType:
            return super(_odict, self).__getitem__(key)
        else:
            raise KeyError, 'Key was not a string or integer'

class DataHolder(ListDict):
    '''
    This class holds data and metadata from a single data file
    
    Arguments:
    data        -- a list of tuples where each tuple is a string, NumPy array pair
    metadata    -- a list of tuples where each tuple is a string, object pair

    Data can be accessed in three ways:
        1. as an attribute
        2. as a dictionary value
        3. as a list item
    Metadata can be accessed in a similar manner though from an attribute called metadata
    '''
    def __init__(self, data, metadata=None):
        ListDict.__init__(self, data)
        self.metadata = ListDict(metadata)
