'''
I/O  package
'''

import os
if os.name == 'java':
    import jython.jycore as _core
    import jython.jyio as _io
else:
    import python.pycore as _core
    import python.pyio as _io

_asIterable = _core.asIterable
_toList = _core.toList
#_asDDict = _core.asDatasetDict
#_asDList = _core.asDatasetList
_iformats = _io.input_formats 
_colourclasses = _io.colour_loaders
_loaders = _io.loaders
_oformats = _io.output_formats
_soformats = _io.scaled_output_formats
_ioexception = _io.io_exception
_srsload = _io.SRSLoader

nxmanager = None

from dictutils import DataHolder

def _findsuffix(name, formats):
    bits = name.split('.')
    if len(bits) > 1:
        suffix = bits[-1].lower()
        if suffix in formats:
            return suffix
    return None

def load(name, formats=None, withmetadata=True, ascolour=False, **kwarg):
    '''Load a file and return a list of datasets (or a dictionary of datasets) and
    optionally a dictionary of metadata items

    formats -- list of formats to try. Supported input formats:
        png, gif, jpeg, tiff
        adsc, crysalis, mar, pilatus -> image detectors
        cbf
        xmap
        srs
        binary -> raw

    '''
    try:
        f = open(name)
        f.close()
    except:
        raise ValueError, 'File %s does not exist' % name
        
    if formats is None:
        # parse name to find extension and match with loader
        suf = _findsuffix(name, _iformats)
        if suf:
            formats = [suf]

    if formats: # remove unsupported
        for f in formats:
            if f not in _iformats:
                formats.remove(f)

    lfh = None
    if not formats:
        # need to attempt to use all supported formats
        for l in _loaders:
            if not l:
                continue

            try:
                if l in _colourclasses:
                    ldr = l(name, not ascolour)
                else:
                    ldr = l(name)
                ldr.setloadmetadata(withmetadata)
                lfh = ldr.load()
                break
            except _ioexception, e:
                print e
                continue
    else:
        for f in formats:
            loader = _iformats[f]

            if not loader:
                continue

            try:
                if loader in _colourclasses:
                    ldr = loader(name, not ascolour)
                else:
                    ldr = loader(name)
                ldr.setloadmetadata(withmetadata)
                lfh = ldr.load()
            except _ioexception, e:
                print e
                continue

    if lfh is None:
        raise _ioexception, "Cannot load file"

    return lfh

def save(name, data, format=None, range=(), autoscale=False):
    '''Save a (list of) datasets with optional scaling range

    Supported output formats:
        png, gif, jpeg, tiff
        binary -> raw
        text -> raw

    '''
    saver = None
    try:
        if len(range) > 0 or autoscale:
            if not autoscale:
                if len(range) != 2:
                    raise ValueError, "Range has to be a pair of limits (lower, upper)"
                if range[0] >= range[1]:
                    raise ValueError, "Given minimum must be less than maximum"
            if format is None:
                format = _findsuffix(name, _soformats)
            print "scaled save format", format
            sclass = _soformats[format]
            if autoscale:
                saver = sclass(name)
            else:
                saver = sclass(name, range[0], range[1])
        else:
            if format is None:
                format = _findsuffix(name, _oformats)
            print "save format", format, "as", name
            sclass = _oformats[format]
            saver = sclass(name)
    except KeyError:
        raise ValueError, "Format not supported"

    dh = DataHolder()
    if format == "binary":
        for d in _asIterable(data):
            dh.append(d.name, d)
    else:
        for d in _toList(data):
            dh.append(d.name, d)
    saver.saveFile(dh)


import os as _os
import os.path as _path
#from scisoftpy import ndarraywrapped as _npwrapped

class srsrun(DataHolder):
    '''Represent a run from an SRS file'''
    def __init__(self, run, datadir=None, ending=".dat"):
        '''Specify a run number (or file name) and data directory

        Arguments:
        run -- integer (negative values mean relative to last run number) or filename
        datadir -- data directory
        ending -- file name ending, defaults to ".dat"
        '''
#        try:
#            self.run = int(run)
#            if run <= 0:
#                try:
#                    import gda.data.NumTracker as NumTracker #@UnresolvedImport
#                    self.run += NumTracker().getCurrentFileNumber()
#                except ImportError:
#                    print "No gda configuration access so cannot support negative numbers"
#                    raise
#
#            if datadir is None:
#                datadir = self._getgdadir()
#            self.srsfile = _path.join(datadir, self.run + ending)
#            if not _os.access(self.srsfile, _os.R_OK):
#                self.srsfile = self._findsrs(datadir, ending)
#        except ValueError:
#            if isinstance(run, str):
#                if run.startswith('/'):
#                    datadir = ""
#                else:
#                    if datadir is None:
#                        datadir = self._getgdadir()
#                self.srsfile = _path.join(datadir, run)
#            else:
#                print "run must be a number or a file path"
        self.run = int(run)
        self.srsfile = self._findsrs(datadir, ending)

        print 'file is', self.srsfile
        self.basedir = _path.dirname(self.srsfile)
        dh = load(self.srsfile, formats=['srs'])
        DataHolder.__init__(self, dh.items(), dh.metadata.items())

    def _getgdadir(self):
        try:
            import gda.data.PathConstructor as PathConstructor
            datadir = PathConstructor.createFromDefaultProperty()
        except ImportError:
            print "No gda configuration access so please specify data directory"
            raise
        return datadir

    def _findsrs(self, datadir, ending=".dat"):
        '''Find an SRS file by looking down two layers from given data directory'''
        fname = "%d%s" % (self.run, ending)
        print 'looking for', fname
        dirs = _os.listdir(datadir)
        dirs = [ _path.join(datadir, d) for d in dirs if _path.isdir(_path.join(datadir, d)) ]
        dirs.sort()
        print dirs
        for d in dirs:
            ldirs = _os.listdir(d)
            ldirs = [ l for l in ldirs if _path.isdir(_path.join(d, l)) ]
            ldirs.sort(reverse=True)
            print ' ', d, ldirs
            for l in ldirs:
                file = _path.join(d, l, fname)
                if _os.access(file, _os.R_OK):
                    return file
        return None

    def __str__(self):
        return "\t".join(self.keys())


if __name__ == '__main__':
    from dictutils import sanitise_name
    insane = ['hello', '1hello', ' hello !!$#' ]
    sane = ['hello', '_1hello', '_hello_____' ]
    for i,s in enumerate(insane):
        ss = sanitise_name(s)
        print "|%s| => |%s| cf |%s|, (%d)" % (s, ss, sane[i], ss == sane[i])

    import sys
    if len(sys.argv) > 1:
        name = sys.argv[1]
    else:
        name = '/scratch/workspace/images/i16pilatus.dat'
    dh = load(name, withmetadata=False)
    from pprint import pprint
    pprint(dh.metadata); print
    pprint(dh); print
