
from uk.ac.diamond.scisoft.analysis.io import PNGLoader as _pngload
from uk.ac.diamond.scisoft.analysis.io import PNGSaver as _pngsave
from uk.ac.diamond.scisoft.analysis.io import PNGScaledSaver as _pngscaledsave
from uk.ac.diamond.scisoft.analysis.io import JPEGLoader as _jpegload
from uk.ac.diamond.scisoft.analysis.io import JPEGSaver as _jpegsave
from uk.ac.diamond.scisoft.analysis.io import JPEGScaledSaver as _jpegscaledsave
from uk.ac.diamond.scisoft.analysis.io import TIFFImageLoader as _tiffload
from uk.ac.diamond.scisoft.analysis.io import TIFFImageSaver as _tiffsave
from uk.ac.diamond.scisoft.analysis.io import JavaImageLoader as _imgload
from uk.ac.diamond.scisoft.analysis.io import JavaImageSaver as _imgsave
from uk.ac.diamond.scisoft.analysis.io import ADSCImageLoader as _adscload

try:
    from uk.ac.diamond.scisoft.analysis.io import CBFLoader as _cbfload
except:
    import sys
    print >> sys.stderr, "Could not import CBF loader"
    print >> sys.stderr, "Problem with path for dynamic/shared library or product bundling"
    _cbfload = None

from uk.ac.diamond.scisoft.analysis.io import CrysalisLoader as _crysload
from uk.ac.diamond.scisoft.analysis.io import MARLoader as _marload
from uk.ac.diamond.scisoft.analysis.io import PilatusTiffLoader as _ptiffload
from uk.ac.diamond.scisoft.analysis.io import SRSLoader as _srsload
from uk.ac.diamond.scisoft.analysis.io import RawBinarySaver as _rawbinsave
from uk.ac.diamond.scisoft.analysis.io import RawBinaryLoader as _rawbinload
from gda.analysis.io import RawOutput as _rawtextsave
from uk.ac.diamond.scisoft.analysis.io import XMapLoader as _xmapload
from gda.analysis.io import ScanFileHolderException as io_exception

from uk.ac.diamond.scisoft.analysis.io import NexusLoader as _nxsloader #@UnresolvedImport
from gda.data.nexus.tree.NexusTreeNodeSelection import SKIP as _skip
from gda.data.nexus.tree.NexusTreeNodeSelection import createTreeForAllNXEntries as _createtree

from uk.ac.diamond.scisoft.analysis.io import NexusManager as _nxsmanager

from jycore import asDatasetDict, asDatasetList, toList

from scisoftpy.dictutils import DataHolder

class nxmanager(object):
    '''This holds a NeXus tree and manages access to it. This provides
    dictionary-like access to lists of datasets
    '''
    def __init__(self, tree):
        '''Arguments:
        tree -- NeXus tree
        '''
        self.tree = tree
        self.mgr = _nxsmanager(tree)
    def __getitem__(self, key):
        '''Return a list of datasets in tree whose names match given key'''
        return asDatasetList(self.mgr.getDatasets(key))

def loadnexus(name):
    '''Load a NeXus file and return a NeXus tree manager'''
    st = _createtree()
    nxloader = _nxsloader(name, _skip, st, None)
    dh = nxloader.loadFile()
    return nxmanager(dh.getNexusTree())

class JavaLoader(object):
    def load(self):
        jdh = self.loadFile()
        data = asDatasetList(jdh.getList())
        names = jdh.getNames()
        if self.load_metadata:
            meta = jdh.getMetadata()
            if len(data) != len(names):
                raise io_exception, "Number of names does not match number of datasets"
            
            if len(meta) > 1:
                print 'File contains more than one metadata maps - using first one'
            if len(meta) == 0:
                metadata = None
            else:
                metadata = [ (k, meta[0][k]) for k in meta[0] ]
        else:
            metadata = None
        return DataHolder(zip(names, data), metadata)

    def setloadmetadata(self, load_metadata):
        self.load_metadata = load_metadata
        self.setLoadMetadata(load_metadata)

class SRSLoader(JavaLoader, _srsload):
    def __init__(self, *arg):
        _srsload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class PNGLoader(JavaLoader, _pngload):
    def __init__(self, *arg):
        _pngload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class JPEGLoader(JavaLoader, _jpegload):
    def __init__(self, *arg):
        _jpegload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class ImageLoader(JavaLoader, _imgload):
    def __init__(self, *arg):
        _imgload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class TIFFLoader(JavaLoader, _tiffload):
    def __init__(self, *arg):
        _tiffload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class ADSCLoader(JavaLoader, _adscload):
    def __init__(self, *arg):
        _adscload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

if _cbfload is None:
    CBFLoader = None
else:
    class CBFLoader(JavaLoader, _cbfload):
        def __init__(self, *arg):
            _cbfload.__init__(self, *arg) #@UndefinedVariable
            self.load_metadata = True

class CrysLoader(JavaLoader, _crysload):
    def __init__(self, *arg):
        _crysload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class MARLoader(JavaLoader, _marload):
    def __init__(self, *arg):
        _marload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class PilLoader(JavaLoader, _ptiffload):
    def __init__(self, *arg):
        _ptiffload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class RawLoader(JavaLoader, _rawbinload):
    def __init__(self, *arg):
        _rawbinload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

class XMapLoader(JavaLoader, _xmapload):
    def __init__(self, *arg):
        _xmapload.__init__(self, *arg) #@UndefinedVariable
        self.load_metadata = True

input_formats = { "png": PNGLoader, "gif": ImageLoader,
               "jpeg": JPEGLoader, "jpg": JPEGLoader,
               "tiff": TIFFLoader, "tif": TIFFLoader,
               "adsc": ADSCLoader, "img": ADSCLoader,
               "cbf": CBFLoader, "crys": CrysLoader,
               "mar": MARLoader, "mccd": MARLoader,
               "pil": PilLoader,
               "srs": SRSLoader,
               "binary": RawLoader, "xmap": XMapLoader #, "nxs": NexusLoader, "h5": NexusLoader
               }
colour_loaders  = [ PNGLoader, ImageLoader, JPEGLoader, TIFFLoader ]
loaders = [ PNGLoader, ADSCLoader, CrysLoader, MARLoader, CBFLoader, XMapLoader, RawLoader, SRSLoader ]

output_formats = { "png": _pngsave, "gif": _imgsave, "jpeg": _jpegsave, "tiff": _tiffsave, "text": _rawtextsave,
              "binary": _rawbinsave }

scaled_output_formats = { "png": _pngscaledsave, "jpeg": _jpegscaledsave }

