'''
'''

from scisoftpy.dictutils import DataHolder

io_exception = IOError

class PythonLoader(object):
    def __init__(self, name, ascolour=False):
        self.ascolour = ascolour
        self.name = name
        self.load_metadata = True

    def setloadmetadata(self, load_metadata):
        self.load_metadata = load_metadata

class SRSLoader(PythonLoader):
    '''
    Loads an SRS dat file and returns a dataholder object
    '''
    def load(self):
        '''
        Returns a DataHolder object
        '''
        f = open(self.name)

        while True:
            l = f.readline().lstrip()
            if l.startswith("&SRS"):
                break
        else:
            raise io_exception, "Not an SRS file"

        srstext = []
        while True:
            l = f.readline()
            ls = l.strip()
            if ls:
                if ls.startswith("&END"):
                    break
                if self.load_metadata:
                    srstext.append(l)
        else:
            raise io_exception, "No end tag found"

        colstext = f.readline().strip()
        datatext = []
        while True:
            l = f.readline()
            if l:
                datatext.append(l)
            else:
                break

        f.close()

        data = self._parse_data(colstext, datatext)
        metadata = self._parse_head(srstext)

        return DataHolder(data, metadata)

    @staticmethod
    def _parse_head(text):
        '''
        Scan header lines and extract key/value pairs. A pair contains an equals sign
        '''
        srsmeta = []
        othermeta = []
        other = False
        for l in text:
            ls = l.strip()
            if ls.startswith("<MetaDataAtStart>"):
                other = True
                continue
            if ls.startswith("</MetaDataAtStart>"):
                other = False
                continue
            if other:
                othermeta.append(ls)
            else:
                srsmeta.append(ls)
        meta = []
        for m in srsmeta:
            ss = SRSLoader._parse_srsline(m)
            for s in ss:
                meta.append(s)
        for m in othermeta:
            s = m.split('=',1)
            if len(s) == 1:
                raise io_exception, "Metadata did not contain equal sign: " + m
            meta.append((s[0], SRSLoader._parse_value(s[1])))
        return meta

    @staticmethod
    def _parse_srsline(line):
        '''
        Scan a SRS header line of comma-separated key/value pairs
        '''
        meta = []
        if line.startswith("SRS"):
            ki = 0
            l = len(line)
            while ki < l:
                kf = line.find('=', ki)
                if kf > 0:
                    k = line[ki:kf]
                    vi = kf+1
                    nc = line.find(',', vi)
                    if nc < vi:
                        raise io_exception, "No comma found: " + line[vi:]
                    qi = line.find('\'', vi)
                    if qi > 0 and qi < nc:
                        if qi != vi:
                            raise io_exception, "Quoted string does not follow equal sign: " + line[qi:]
                        # inside quoted string
                        qf = line.find('\'', qi+1)
                        if qf > 0 and qf != (nc-1):
                            raise io_exception, "A comma does not follow a quoted string: " + line[qi:]

                    vf = nc
                    v = line[vi:vf]
                    meta.append((k, SRSLoader._parse_value(v)))
                    ki = nc+1
                else:
                    raise io_exception, "No equal sign on line: " + line[ki:]

        return meta

    @staticmethod
    def _parse_data(cols, text):
        '''
        Convert to all data to dictionary. Keys are column headers and values are
        1D NumPy arrays 
        '''
        cols = cols.split('\t')
        lc = len(cols)
        data = [[] for dummy in cols]
        for t in text:
            r = t.split('\t')
            for i,v in enumerate(r):
                data[i].append(SRSLoader._parse_value(v))
            lr = len(r)
            if lr < lc:
                print 'Short row!'
                for i in range(lr, lc):
                    data[i].append(0)

        from pycore import array as parray
        return [(c, parray(d)) for c, d in zip(cols, data)]

    @staticmethod
    def _parse_value(text):
        '''
        Convert to integer, float or string (strips outer quotes)
        '''
        try:
            v = int(text)
        except ValueError:
            try:
                v = long(text)
            except ValueError:
                try:
                    v = float(text)
                except ValueError:
                    if text.startswith('\''):
                        if text.endswith('\''):
                            v = text[1:-1]
                        else:
                            raise io_exception, "No matching single quotes in string: " + text
                    if text.startswith('"'):
                        if text.endswith('"'):
                            v = text[1:-1]
                        else:
                            raise io_exception, "No matching double quotes in string: " + text
        return v

import pycore as _core

import Image as _im #@UnresolvedImport

class ImageLoader(PythonLoader):
    def load(self):
        im = _im.open(self.name)
        if im.mode == 'RGB':
            if self.ascolour:
                # convert to an rgb dataset
                pass
            else:
                im = im.convert('L')
        d = _core.asarray(im)
        return DataHolder([(self.name, d)])
'''
import FreeImagePy

fi = FreeImagePy.Image(libraryName='/dls_sw/apps/python/FreeImage/lib/libfreeimage.so')

fi.load('/scratch/workspace/images/test.png')

(fi.size, fi.format, fi.bpp, fi.metadata)
pim = fi.convertToPil()
fi.close()

import numpy as np

im = np.asarray(pim)

'''

import libtiff as _lt #@UnresolvedImport


class LibTIFFLoader(PythonLoader):
    def load(self):
        t = _lt.TIFF.open(self.name)
        data = [ ("image%d" % i, im) for i,im in enumerate(t.iter_images()) ]
        im = t.read_image()
        metadata = None
        if self.load_metadata:
            metatext = t.info()
            t.close()

            import re
            ra = re.compile('(\w+): (.+)', re.DOTALL)
            b = 0
            k = None
            metadata = []
            s = ra.search(metatext, b)
            while s is not None:
                i = s.start(1)
                if i > 0:
                    v = metatext[b:i]
                    metadata.append((k,v))
            
                k = s.group(1)
                b = s.start(2)
                s = ra.search(metatext, b)
            
            v = metatext[b:]
            metadata.append((k,v))
        return DataHolder(data, metadata)

'''
import libtiff as lt
t = lt.TIFF.open('/scratch/workspace/images/test.tiff')
im = t.read_image()
metatext = t.info()
t.close()

ra = re.compile('(\w+): (.+)', re.DOTALL)
b = 0
d = []
s = ra.search(metatext, b)
while s is not None:
    i = s.start(1)
    if i > 0:
        v = metatext[b:i]
        d.append((k,v))

    k = s.group(1)
    b = s.start(2)
    s = ra.search(metatext, b)

v = metatext[b:]
d.append((k,v))

'''

PNGLoader = ImageLoader
JPEGLoader = ImageLoader
TIFFLoader = LibTIFFLoader

ADSCLoader = None
CBFLoader = None
CrysLoader = None
MARLoader = ImageLoader
PilLoader = LibTIFFLoader
RawLoader = None
XMapLoader  = None

_pngsave = None
_jpegsave = None
_tiffsave = None
_imgsave = None
_rawtextsave = None
_rawbinsave = None
_pngscaledsave = None
_jpegscaledsave = None

from pycore import asDatasetDict, asDatasetList, toList

input_formats = { "png": PNGLoader, "gif": ImageLoader,
               "jpeg": JPEGLoader, "jpg": JPEGLoader,
               "tiff": TIFFLoader, "tif": TIFFLoader,
               "adsc": ADSCLoader, "img": ADSCLoader,
               "cbf": CBFLoader, "crys": CrysLoader,
               "mar": MARLoader, "mccd": MARLoader,
               "pil": PilLoader,
               "srs": SRSLoader,
               "binary": RawLoader, "xmap": XMapLoader #, "nxs": NexusLoader, "h5": NexusLoader
               }
colour_loaders  = [ PNGLoader, ImageLoader, JPEGLoader, TIFFLoader ]
loaders = [ PNGLoader, ADSCLoader, CrysLoader, MARLoader, CBFLoader, XMapLoader, RawLoader, SRSLoader ]

output_formats = { "png": _pngsave, "gif": _imgsave, "jpeg": _jpegsave, "tiff": _tiffsave, "text": _rawtextsave,
              "binary": _rawbinsave }

scaled_output_formats = { "png": _pngscaledsave, "jpeg": _jpegscaledsave }

